#include <ArduinoBLE.h>

BLEService bleServiceTest("776982a9-8315-4bad-80f7-325e6a521aa6");
BLECharacteristic bleCharacteristicSwitch("df172997-36a6-446c-b70c-ac1556012427", BLERead | BLEWrite, 32);

bool iBLEState = 0;

void bleOnConnected(BLEDevice bleDevice)
{
  Serial.println("bleOnConnected: bleDevice.address()=" + bleDevice.address());
  Serial.println("BLE.stopAdvertise()");
  BLE.stopAdvertise();  //no effect on XIAO_ESP32S3
}

void bleOnDisconnected(BLEDevice bleDevice)
{
  iBLEState = 2;
  Serial.println("bleOnDisconnected: bleDevice.address()=" + bleDevice.address());
}

void bleOnCharacteristicStringWritten(BLEDevice bleDevice, BLECharacteristic characteristic)
{
  String sWritten = String (((char*)characteristic.value()), characteristic.valueLength());
  Serial.println("bleOnCharacteristicStringWritten: " + sWritten);
}

void bleOnCharacteristicStringRead(BLEDevice bleDevice, BLECharacteristic characteristic)
{
  characteristic.setValue(String("<123454321>").c_str());
  String sRead = String (((char*)characteristic.value()), characteristic.valueLength());
  Serial.println("bleOnCharacteristicStringRead: " + sRead);
}

void setup() 
{
  Serial.begin(115200);
  while (!Serial) 
    delay(10);

  Serial.println("setup: calling BLE.begin()");
  if (!BLE.begin())
    Serial.println("BLE.begin() failed");
  iBLEState = 1;
  BLE.setLocalName("BLE Server Test");
  BLE.setAdvertisedService(bleServiceTest);
  bleServiceTest.addCharacteristic(bleCharacteristicSwitch);
  BLE.addService(bleServiceTest);
  BLE.setEventHandler(BLEConnected, bleOnConnected);
  BLE.setEventHandler(BLEDisconnected, bleOnDisconnected);
  
  bleCharacteristicSwitch.setEventHandler(BLEWritten, bleOnCharacteristicStringWritten);
  bleCharacteristicSwitch.setEventHandler(BLERead, bleOnCharacteristicStringRead);
  bleCharacteristicSwitch.setValue(String("INIT").c_str());
  
  BLE.advertise();
}

void loop() 
{
  static uint32_t uLoopCount = 0;
  uLoopCount++;
  
  if (uLoopCount == 1)
    Serial.println("loop: first call"); 
  if (uLoopCount % 100  == 0)
    Serial.println("loop: uLoopCount=" + String(uLoopCount)); 
  
  // poll for Bluetooth® Low Energy events
  if (iBLEState == 1)
    BLE.poll();
  else if (iBLEState == 2)
  {
    iBLEState = 3;
    //Serial.println("BLE.stopAdvertise()");
    //BLE.stopAdvertise();  //no effect on XIAO_ESP32S3
    Serial.println("BLE.end()");  //never called
    BLE.end();
    Serial.println("start delay(2000)");
    delay(2000);
    Serial.println("end delay(2000)");
  }

  delay(10);
}